// New style comment marker used to force error in 16-bit dBW/VdB
//---------------------------------------------------------------------
// Program......: VdBWin32.PRG
// Programmer...: Jim Sare (jimsare@ameritech.net)
// Last Modified: 04/11/1998
// Rev. History.: 08/10/1995 -- Added NavFileType, FindWinByText,
//              :               FindWinByClass.
//              :               Changed FindVdBWin to use FindWinByText
//              :               and FindWinByClass.
//              :               Made compatible with VdB and Win95.
//              : 08/27/1995 -- Added WinRestore, updated comments.
//              : 04/11/1998 -- Initial adaptation for 32-bit.
// Notes........: Contains routines to:
//              : Minimize, maximize, restore, hide, & show windows.
//              : Determine hWND of any dBW/VdB child window.
//              : Find a child window of a parent window by text or classname.
//              : Select the filetype displayed in Navigator.
// Written for..: Visual dBASE (32-bit versions)
// Useage.......: Set Procedure To VdBWin32.PRG Additive
//
// Extended useage notes follow:
//
// Use FindVdBWin(WindowCaption) to obtain the hWND of a dBW child window.
// The WindowCaption parameter is not case sensitive.
// FindVdBWin() returns 0 if the dBW child window could not be found.
// FindVdBWin() will only search for dBW child windows. It will not find
// windows that are not dBW child windows.
//
//       hWND = FindVdBWin("Navigator")  // Returns the Navigator window handle.
//
// The functions WinRestore, WinNormal, WinMin, WinMax, WinHide, and WinShow
// should work with non-dBW windows in addition to dBW/VdB child windows. The
// parameter to these functions is a valid window handle.
//
// Place the following 3 lines at the beginning of a DOS dBASE application
// to minimize the Navigator and maximize the Program window.
//
//       Set Procedure To dBWWin.prg Additive
//       WinHide(FindVdBWin("navigator"))
//       WinMax(FindVdBWin("program"))
//
// This is where your application runs.
//
// Place the following 3 lines at the very end of your application to
// restore the Navigator and Command windows to their normal states and
// close the dBWWin procedure file.
//
//       WinShow(FindVdBWin("navigator"))
//       WinNormal(FindVdBWin("program"))
//       Close Procedure dBWWin.prg
//
// Additionally, the procedures in this file can be used in conjunction with
// the procedure FindApp in the file AppMgr.PRG to control desktop application
// windows. For example, to minimize Program Manager, place the following line
// in your application:
//
//       WinMin(FindApp("Program Manager"))
//
// To restore the Program Manager window to the state it was in before it was
// minimized:
//
//       WinRestore(FindApp("Program Manager"))
//---------------------------------------------------------------------

FUNCTION WinRestore(nHandle)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Restore a window to the state it was in before being
//             : minimized.
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 10/23/1994 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: None
//             : WINAPI: IsWindow, ShowWindow
// Called by...: Any
// Usage.......: WinRestore(nHandle)
// Example.....: WinRestore(FindVdBWin("Navigator"))
// Returns.....: .T. if parameter is valid window handle, .F. otherwise.
// Parameters..: nHandle = Handle of the window to restore.
//---------------------------------------------------------------------

	If Type("IsWindow") # "FP"
		extern CLOGICAL IsWindow(CHANDLE) User32
	EndIf
	If Type("ShowWindow") # "FP"
		extern CLOGICAL ShowWindow(CHANDLE, CINT) User32
	EndIf
		#ifndef SW_RESTORE
			#define SW_RESTORE			9
		#endif

	If IsWindow(nHandle)
		ShowWindow(nHandle, SW_RESTORE)
	EndIf
	RETURN IsWindow(nHandle)				// .T. if valid window handle, .F. otherwise
// EOP: WinRestore

FUNCTION WinNormal(nHandle)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Set a window to its normal size.
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 10/23/1994 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: None
//             : WINAPI: IsWindow, ShowWindow
// Called by...: Any
// Usage.......: WinNormal(nHandle)
// Example.....: WinNormal(FindVdBWin("Navigator"))
// Returns.....: .T. if parameter is valid window handle, .F. otherwise.
// Parameters..: nHandle = Handle of the window to switch to normal size.
//---------------------------------------------------------------------

	If Type("IsWindow") # "FP"
		extern CLOGICAL IsWindow(CHANDLE) User32
	EndIf
	If Type("ShowWindow") # "FP"
		extern CLOGICAL ShowWindow(CHANDLE, CINT) User32
	EndIf
		#ifndef SW_SHOWNORMAL
			#define SW_SHOWNORMAL		1
		#endif

	If IsWindow(nHandle)
		ShowWindow(nHandle, SW_SHOWNORMAL)
	EndIf
	RETURN IsWindow(nHandle)				// .T. if valid window handle, .F. otherwise
// EOP: WinNormal

FUNCTION WinMin(nHandle)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Set a window to its minimized state.
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 10/23/1994 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: None
//             : WINAPI: IsWindow, ShowWindow
// Called by...: Any
// Usage.......: WinMin(nHandle)
// Example.....: WinMin(FindVdBWin("Navigator"))
// Returns.....: .T. if parameter is valid window handle, .F. otherwise.
// Parameters..: nHandle = Handle of the window to switch to minimized state.
//---------------------------------------------------------------------

	If Type("IsWindow") # "FP"
		extern CLOGICAL IsWindow(CHANDLE) User32
	EndIf
	If Type("ShowWindow") # "FP"
		extern CLOGICAL ShowWindow(CHANDLE, CINT) User32
	EndIf
		#ifndef SW_SHOWNIMNOACTIVE
			#define SW_SHOWMINNOACTIVE	7
		#endif

	If IsWindow(nHandle)
		ShowWindow(nHandle, SW_SHOWMINNOACTIVE)
	EndIf
	RETURN IsWindow(nHandle)				// .T. if valid window handle, .F. otherwise
// EOP: WinMin

FUNCTION WinMax(nHandle)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Set a window to its maximized state.
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 10/23/1994 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: None
//             : WINAPI: IsWindow, ShowWindow
// Called by...: Any
// Usage.......: WinMax(nHandle)
// Example.....: WinMax(FindVdBWin("Navigator"))
// Returns.....: .T. if parameter is valid window handle, .F. otherwise.
// Parameters..: nHandle = Handle of the window to switch to maximized state.
//---------------------------------------------------------------------

	If Type("IsWindow") # "FP"
		extern CLOGICAL IsWindow(CHANDLE) User32
	EndIf
	If Type("ShowWindow") # "FP"
		extern CLOGICAL ShowWindow(CHANDLE, CINT) User32
	EndIf
		#ifndef SW_SHOWMAXIMIZED
			#define SW_SHOWMAXIMIZED	3
		#endif

	If IsWindow(nHandle)
		ShowWindow(nHandle, SW_SHOWMAXIMIZED)
	EndIf
	RETURN IsWindow(nHandle)				// .T. if valid window handle, .F. otherwise
// EOP: WinMax

FUNCTION WinHide(nHandle)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Hide a window (ie; Makes a window invisible).
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 10/23/1994 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: None
//             : WINAPI: IsWindow, ShowWindow
// Called by...: Any
// Usage.......: WinHide(nHandle)
// Example.....: WinHide(FindVdBWin("Navigator"))
// Returns.....: .T. if parameter is valid window handle, .F. otherwise.
// Parameters..: nHandle = Handle of the window to hide.
//---------------------------------------------------------------------

	If Type("IsWindow") # "FP"
		extern CLOGICAL IsWindow(CHANDLE) User32
	EndIf
	If Type("ShowWindow") # "FP"
		extern CLOGICAL ShowWindow(CHANDLE, CINT) User32
	EndIf
		#ifndef SW_HIDE
			#define SW_HIDE				0
		#endif

	If IsWindow(nHandle)
		ShowWindow(nHandle, SW_HIDE)
	EndIf
	RETURN IsWindow(nHandle)				// .T. if valid window handle, .F. otherwise
// EOP: WinHide

FUNCTION WinShow(nHandle)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Display a hidden window. (ie; Makes a window visible)
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 10/23/1994 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: None
//             : WINAPI: IsWindow, ShowWindow
// Called by...: Any
// Usage.......: WinShow(nHandle)
// Example.....: WinShow(FindVdBWin("Navigator"))
// Returns.....: .T. if parameter is valid window handle, .F. otherwise.
// Parameters..: nHandle = Handle of the window to show.
//---------------------------------------------------------------------

	If Type("IsWindow") # "FP"
		extern CLOGICAL IsWindow(CHANDLE) User32
	EndIf
	If Type("ShowWindow") # "FP"
		extern CLOGICAL ShowWindow(CHANDLE, CINT) User32
	EndIf
		#ifndef SW_SHOW
			#define SW_SHOW				5
		#endif

	If IsWindow(nHandle)
		ShowWindow(nHandle, SW_SHOW)
	EndIf
	RETURN IsWindow(nHandle)				// .T. if valid window handle, .F. otherwise
// EOP: WinShow

FUNCTION FindVdBWin(cWindowCaption)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Find a dBW/VdB MDI child window by its window text.
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 10/23/1994 -- Original
//             : 08/10/1995 -- Modified to use FindChildByClass and
//             :               FindChildByText
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: FindChildByClass, FindChildByText
// Called by...: Any
// Usage.......: FindVdBWin(cWindowCaption)
// Example.....: nHandle = FindVdBWin("Navigator")
// Returns.....: Handle of the window if found, 0 otherwise.
// Parameters..: cWindowCaption = Title bar text of window. Not case
//             : sensitive. Will find on partial text.
//---------------------------------------------------------------------
	LOCAL nThehWND

	nThehWND = FindChildByClass(_app.FrameWin.hWND, "MDICLIENT")
	If nThehWND = 0
		RETURN 0									// If not found, bad news
	EndIf
	RETURN FindChildByText(nThehWND, cWindowCaption)	// # 0 on success, 0 otherwise
// EOP: FindVdBWin

FUNCTION NavFileType(cFileType)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Change the filetype displayed in the Navigator. This is
//             : useful in a StartUp.PRG to default to a filetype other
//             : than Table.
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 08/10/1995 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: FindVdBWin, FindChildByClass
//             : WINAPI: GetDlgCtrlID, SendMessage
// Called by...: Any
// Usage.......: NavFileType(cFileType)
// Example.....: NavFileType("forms"))
// Returns.....: .T. if successful, .F. otherwise.
// Parameters..: cFileType = Type of file to display in the Navigator.
//---------------------------------------------------------------------
	LOCAL hNav, nThehWND, nIndex, aTemp

	If Type("GetWindow") # "FP"
		extern CHANDLE GetWindow(CHANDLE, CUINT) User32
	EndIf
		#ifndef GW_CHILD
			#define GW_CHILD				5
		#endif
	If Type("SendMessage") # "FP"
		extern CLONG SendMessage(CHANDLE, CUINT, CLONG, CLONG) User32;
					From "SendMessageA"
	EndIf
		#ifndef TCM_FIRST
			#define TCM_FIRST				0x1300      // Tab control messages
		#endif
		#ifndef TCM_SETCURSEL
			#define TCM_SETCURSEL		TCM_FIRST + 12
		#endif
		#ifndef VDBM_VIEWREFRESH
			#define VDBM_VIEWREFRESH	0x0982
		#endif
		#ifndef WM_COMMAND
			#define WM_COMMAND			0x0111
		#endif

	hNav = FindVdBWin("NAVIGATOR")			// Find Navigator handle
	If hNav = 0
		RETURN .F.
	EndIf
	nThehWND = GetWindow(hNav, GW_CHILD)
	nThehWND = FindChildByClass(nThehWND, "SysTabControl32")
	If nThehWND = 0
		RETURN .F.
	EndIf
	aTemp = {"ALL", "PROJECTS", "FORMS", "REPORTS", "PROGRAMS", "TABLES", "SQL", "DATA MODULES", "IMAGES", "OTHER"}
	nIndex = aTemp.Scan(Upper(cFileType))
	If nIndex > 0							// If the string was not found and selected
		SendMessage(nThehWND, TCM_SETCURSEL, nIndex - 1, 0)
		SendMessage(hNav, WM_COMMAND, VDBM_VIEWREFRESH, 0)
	EndIf
	RETURN nIndex # 0
// EOP: NavFileType

FUNCTION NavFont(nFont)
	LOCAL hNav, nThehWND, hWND, hFont

	If Type("GetStockObject") # "FP"
		extern CHANDLE GetStockObject(CINT) GDI32
	EndIf
		#ifndef OEM_FIXED_FONT
			#define OEM_FIXED_FONT			10
		#endif
		#ifndef ANSI_FIXED_FONT
			#define ANSI_FIXED_FONT			11
		#endif
		#ifndef ANSI_VAR_FONT
			#define ANSI_VAR_FONT			12
		#endif
		#ifndef SYSTEM_FONT
			#define SYSTEM_FONT				13
		#endif
		#ifndef DEVICE_DEFAULT_FONT
			#define DEVICE_DEFAULT_FONT	14
		#endif
		#ifndef SYSTEM_FIXED_FONT
			#define SYSTEM_FIXED_FONT		16
		#endif
		#ifndef DEFAULT_GUI_FONT
			#define DEFAULT_GUI_FONT		17
		#endif
	If Type("GetWindow") # "FP"
		extern CHANDLE GetWindow(CHANDLE, CUINT) User32
	EndIf
		#ifndef GW_HWNDNEXT
			#define GW_HWNDNEXT				2
		#endif
		#ifndef GW_CHILD
			#define GW_CHILD					5
		#endif
	If Type("SendMessage") # "FP"
		extern CLONG SendMessage(CHANDLE, CUINT, CLONG, CLONG) User32;
					From "SendMessageA"
	EndIf
		#ifndef WM_SETFONT
			#define WM_SETFONT				0x0030
		#endif
	If (nFont < 10) Or (nFont = 15) Or (nFont > 17)
		RETURN False
	EndIf
	hNav = FindVdBWin("Navigator")
	If hNav = 0
		RETURN False
	EndIf
	hFont = GetStockObject(nFont)
	nThehWND = GetWindow(hNav, GW_CHILD)
	nThehWND = GetWindow(nThehWND, GW_CHILD)
	Do While nThehWND # 0
		SendMessage(nThehWND, WM_SETFONT, hFont, 1)
		hWND = GetWindow(nThehWND, GW_CHILD)
		Do While hWND # 0
			SendMessage(hWND, WM_SETFONT, hFont, 1)
			hWND = GetWindow(hWND, GW_HWNDNEXT)
		EndDo
		nThehWND = GetWindow(nThehWND, GW_HWNDNEXT)
	EndDo
	RETURN True

// EOP: NavFont

FUNCTION FindChildByText(nHandle, cWinText)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Find by window text a child window of the specified
//             : parent window. Can be used to find controls as well
//             : as windows.
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 08/10/1995 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: None
//             : WINAPI: GetWindowText, GetWindow
// Called by...: Any
// Usage.......: FindChildByText(nHandle, cText)
// Example.....: hChild = FindChildByText(MyHandle, "WindowText")
// Returns.....: Handle of window if found, 0 otherwise.
// Parameters..: nHandle = Handle of the parent window.
//             : cText = Text of the window to find.
//---------------------------------------------------------------------
	LOCAL nThehWND, xBUFF, nLen, cText, nInd

	If Type("GetWindowText") # "FP"
		extern CINT GetWindowText(CHANDLE, CPTR, CINT) User32;
				From "GetWindowTextA"
	EndIf
	If Type("GetWindow") # "FP"
		extern CHANDLE GetWindow(CHANDLE, CUINT) User32
	EndIf
		#ifndef GW_HWNDNEXT
			#define GW_HWNDNEXT			2
		#endif
		#ifndef GW_CHILD
			#define GW_CHILD				5
		#endif

	nThehWND = GetWindow(nHandle, GW_CHILD)	// Get first child win of nHandle
	Do While nThehWND # 0							// Search child wins until cText found
		xBUFF = Space(41)
		nLen = GetWindowText(nThehWND, xBUFF, 80)
		cText = ""
		For nInd = 0 To nLen - 1
			cText += Chr(xBUFF.GetByte(nInd))
		EndFor
		If Upper(cWinText) $ Upper(OEM(cText))
			Exit
		EndIf
		nThehWND = GetWindow(nThehWND, GW_HWNDNEXT)
	EndDo
	RETURN nThehWND									// hWND if found, 0 if not found
// EOP: FindChildByText

FUNCTION FindChildByClass(nHandle, cClassName)
//---------------------------------------------------------------------
// Programmer..: Jim Sare (CIS: 71562,413)
// Date........: 08/27/1995
// Notes.......: Find by window class a child window of the specified
//             : parent window. Can be used to find controls as well
//             : as windows.
// Written for.: Visual dBASE (32-bit versions)
// Rev. History: 08/10/1995 -- Original
//             : 08/27/1995 -- General code & comment cleanup.
//             : 04/11/1998 -- Initial adaptation for 32-bit.
// Calls.......: None
//             : WINAPI: GetClassName, GetWindow
// Called by...: Any
// Usage.......: FindChildByClass(nHandle, cText)
// Example.....: hChild = FindChildByClass(_app.FrameWin.hWND, "MDIClient")
// Returns.....: Handle of window if found, 0 otherwise.
// Parameters..: nHandle = Handle of the parent window.
//             : cClassName = ClassName of the window to find.
//---------------------------------------------------------------------
	LOCAL nThehWND, xBUFF, nLen, cText, nInd

	If Type("GetClassName") # "FP"
		extern CINT GetClassName(CHANDLE, CPTR, CINT) User32;
				From "GetClassNameA"
	EndIf
	If Type("GetWindow") # "FP"
		extern CHANDLE GetWindow(CHANDLE, CUINT) User32
	EndIf
		#ifndef GW_HWNDNEXT
			#define GW_HWNDNEXT			2
		#endif
		#ifndef GW_CHILD
			#define GW_CHILD				5
		#endif

	nThehWND = GetWindow(nHandle, GW_CHILD)	// Get first child win of nHandle
	Do While nThehWND # 0							// Search child wins until cClassName found
		xBUFF = Space(41)
		nLen = GetClassName(nThehWND, xBUFF, 80)
		cText = ""
		For nInd = 0 To nLen - 1
			cText += Chr(xBUFF.GetByte(nInd))
		EndFor
		If Upper(cClassName) $ Upper(OEM(cText))
			Exit
		EndIf
		nThehWND = GetWindow(nThehWND, GW_HWNDNEXT)
	EndDo
	RETURN nThehWND									// hWND if found, 0 if not found
// EOP: FindChildByClass
// EOF VdBWin32.PRG

