***************************************************************************
*                                                                         *
*  Copyright (C) 1998 Marcello Perathoner. All rights reserved.           *
*                                                                         *
*  Marcello Perathoner, Hard- und Software-Entwicklung, Spatenstr. 33,    *
*  40470 Duesseldorf, Germany                                             *
*                                                                         *
***************************************************************************

************************************************************************
* some example code
************************************************************************

   LOCAL oPrinter ; oPrinter = new DirectPrinterAccess ()

#ifdef __vdb__
   * change the printer name to one you have
   *** the 32-bit class requires the PRINTER NAME ***
   oPrinter.OpenDocument ("HP LaserJet III", "My Test Document")
#else
   *** the 16-bit class requires the PORT ***
   oPrinter.OpenDocument ("LPT1:", "My Test Document")
#endif

   LOCAL esc ; esc = chr (27)

   ************************************************************************
   * PCL escape codes
   *
   * these work only on a PCL printer.
   * if you are using another type of printer you must
   * change these.
   *
   * tip: if you put these (and others you may need)
   * into a database you can easily configure printer types.
   *
   LOCAL italic      ; italic       = esc + "(s1S"
   LOCAL roman       ; roman        = esc + "(s0S"
   LOCAL compressed  ; compressed   = esc + "&k2S"
   LOCAL elite       ; elite        = esc + "&k4S"
   *
   ************************************************************************

   oPrinter.Print     (elite)
   oPrinter.Print     ("Hello, ")
   oPrinter.Print     ("World!")
   oPrinter.PrintLine ()

   oPrinter.PrintLine ("Hello1")
   oPrinter.PrintLine (italic + "Hello2" + roman)
   oPrinter.PrintLine ("Hello3")
   oPrinter.PrintLine (compressed + "Hello4" + elite)
   oPrinter.PrintLine ("Hello5")

   * always eject when you are done or printer will wait
   * for more input
   oPrinter.Eject ()

   oPrinter.CloseDocument ()
   ? "ErrMsg = ", oPrinter.cErrMsg
RETURN

#ifndef __vdb__

************************************************************************
* the Win16 class
************************************************************************

#define HPJOB  CHANDLE
#define HDC    CHANDLE
#define int    CINT
#define LPSTR  CSTRING
#define false  .f.
#define true   .t.

* spooler error codes

#define SP_NOTREPORTED     0x4000   && set if GDI did not report errors
#define SP_ERROR           (-1)     && general error
#define SP_APPABORT        (-2)     && app aborted the job - callback function returned false
#define SP_USERABORT       (-3)     && user aborted the job through spooler's front end
#define SP_OUTOFDISK       (-4)     && not enough disk space to spool
#define SP_OUTOFMEMORY     (-5)     && not enough memory

CLASS DirectPrinterAccess
   extern   HPJOB   WINAPIOpenJob           (LPSTR, LPSTR, HDC)      GDI from "OpenJob"
   extern   int     WINAPIStartSpoolPage    (HPJOB)                  GDI from "StartSpoolPage"
   extern   int     WINAPIEndSpoolPage      (HPJOB)                  GDI from "EndSpoolPage"
   extern   int     WINAPIWriteSpool        (HPJOB, LPSTR, int)      GDI from "WriteSpool"
   extern   int     WINAPICloseJob          (HPJOB)                  GDI from "CloseJob"
   extern   int     WINAPIDeleteJob         (HPJOB, int)             GDI from "DeleteJob"
   extern   int     WINAPIWriteDialog       (HPJOB, LPSTR, int)      GDI from "WriteDialog"
   extern   int     WINAPIDeleteSpoolPage   (HPJOB)                  GDI from "DeleteSpoolPage"
   this.hJob = 0
   this.nErrCode = 0
   this.cErrMsg = ""

   PROCEDURE OpenDocument (cPort, cDocumentName)
      IF this.hJob <> 0
         RETURN false
      ENDIF
      this.hJob = WinAPIOpenJob (cPort, substr (cDocumentName, 1, 32), 0)
      IF this.hJob <= 0
         this.nErrCode = this.hJob
         this.cErrMsg = "OpenJob error"
         RETURN .f.
      ENDIF
      this.nErrCode = WinAPIStartSpoolPage (this.hJob)
      IF this.nErrCode <= 0
         this.cErrMsg = "StartSpoolPage error"
         WinAPIDeleteJob (this.hJob, 0)
         this.hJob = 0
         RETURN false
      ENDIF
   RETURN true

   PROCEDURE Print (cData)
      IF this.hJob == 0
         RETURN false
      ENDIF
      LOCAL BytesWritten
      BytesWritten = WinAPIWriteSpool (this.hJob, cData, len (cData))
      IF BytesWritten <> len (cData)
         this.nErrCode = BytesWritten
         this.cErrMsg = "WriteSpool error"
         WinAPIEndSpoolPage (this.hJob)
         WinAPIDeleteJob (this.hJob, 0)
         this.hJob = 0
         RETURN false
      ENDIF
   RETURN true

   PROCEDURE CloseDocument
      IF this.hJob == 0
         RETURN false
      ENDIF
      this.nErrCode = WinAPIEndSpoolPage (this.hJob)
      IF this.nErrCode <= 0
         this.cErrMsg = "EndSpoolPage error"
         WinAPIDeleteJob (this.hJob, 0)
         this.hJob = 0
         RETURN false
      ENDIF
      this.nErrCode = WinAPICloseJob (this.hJob)
      IF this.nErrCode <= 0
         this.cErrMsg = "CloseJob error"
         WinAPIDeleteJob (this.hJob, 0)
         this.hJob = 0
         RETURN false
      ENDIF
      this.hJob = 0
   RETURN true

   PROCEDURE AbortDocument
      IF this.hJob == 0
         RETURN false
      ENDIF
      this.nErrCode = WinAPIDeleteJob (this.hJob, 0)
      IF this.nErrCode <= 0
         this.cErrMsg = "DeleteJob error"
         this.hJob = 0
         RETURN false
      ENDIF
      this.hJob = 0
   RETURN true

   PROCEDURE PrintLine (cData)
      IF pcount () >= 1
         this.Print (cData)
      ENDIF
      this.Print (chr (13) + chr (10))
   RETURN

   PROCEDURE Eject
      this.Print (chr (12))
   RETURN
ENDCLASS

#else

************************************************************************
* the Win32 class
************************************************************************

#define  LPASTR    CSTRING
#define  LPWSTR    CPTR
#define  LPHANDLE  CPTR CUINT
#define  LPSTRUCT  CPTR
#define  DWORD     CUINT
#define  LPDWORD   CPTR CUINT
#define  HANDLE    CHANDLE
#define  BOOL      CLOGICAL

CLASS DirectPrinterAccess
   extern BOOL    WinAPIOpenPrinter       (LPASTR, LPHANDLE, LPSTRUCT)     winspool.drv  from "OpenPrinterA"
   extern DWORD   WinAPIStartDocPrinter   (HANDLE, DWORD, LPSTRUCT)        winspool.drv  from "StartDocPrinterA"
   extern BOOL    WinAPIStartPagePrinter  (HANDLE)                         winspool.drv  from "StartPagePrinter"
   extern BOOL    WinAPIEndPagePrinter    (HANDLE)                         winspool.drv  from "EndPagePrinter"
   extern BOOL    WinAPIEndDocPrinter     (HANDLE)                         winspool.drv  from "EndDocPrinter"
   extern BOOL    WinAPIClosePrinter      (HANDLE)                         winspool.drv  from "ClosePrinter"
   extern BOOL    WinAPIWritePrinter      (HANDLE, LPASTR, DWORD, LPDWORD) winspool.drv  from "WritePrinter"
   extern DWORD   WinAPIGetLastError      ()                               kernel32      from "GetLastError"
   this.hPrinter = 0
   this.cErrMsg  = ""

   * cPrinterName  : Name of the printer eg. "HP LaserJet III"
   * cDocumentName : Name of the document max 32 chars eg. "Visual dBase monthly bug report"

   PROCEDURE OpenDocument (cPrinterName, cDocumentName)
      IF this.hPrinter <> 0
         RETURN false
      ENDIF
      this.cPrinterName  = cPrinterName
      this.cDocumentName = cDocumentName.substring (0, 31)  * max 32. chars

      * First get a handle to the printer
      IF .not. WinAPIOpenPrinter (this.cPrinterName, this.hPrinter, 0)
         this.cErrMsg = "OpenPrinter error"
         RETURN false
      ENDIF

      * Fill in the structure with info about the document
      LOCAL DocInfo  ; DocInfo  = space (8)           * 8 unicode spaces = 16 bytes
      LOCAL DocName  ; DocName  = this.UniCode2Ansi (this.cDocumentName)
      LOCAL DataType ; DataType = this.UniCode2Ansi ("RAW")
      * You are not expected to understand this <g>
      * This declaration is deliberately wrong
      extern BOOL _WinAPISetRect (CPTR, CPTR, CPTR, CPTR, CPTR) user32 from "SetRect"
      _WinAPISetRect (DocInfo, DocName, 0, DataType, 0)
      _WinAPISetRect = NULL

      * Inform the spooler the document is beginning
      IF WinAPIStartDocPrinter (this.hPrinter, 1, DocInfo) == 0
         this.cErrMsg = "StartDocPrinter error"
         WinAPIClosePrinter (this.hPrinter)
         this.hPrinter = 0
         RETURN false
      ENDIF

      * Start a page
      IF .not. WinAPIStartPagePrinter (this.hPrinter)
         this.cErrMsg = "StartPagePrinter error"
         WinAPIEndDocPrinter (this.hPrinter)
         WinAPIClosePrinter (this.hPrinter)
         this.hPrinter = 0
         RETURN false
      ENDIF
   RETURN

   PROCEDURE Unicode2Ansi (s)
      LOCAL len ; len = s.length
      LOCAL r ; r = space (len / 2 + 1)
      LOCAL i
      FOR i = 0 to len - 1
         r.setbyte (i, asc (s.substring (i, len)))
      NEXT
      r.setbyte (len, 0)
   RETURN r

   PROCEDURE Print (cData)
      IF this.hPrinter = 0
         RETURN false
      ENDIF

      * Send the data to the printer
      LOCAL nBytesWritten ; nBytesWritten = 0
      IF .not. WinAPIWritePrinter (this.hPrinter, cData, cData.length, nBytesWritten)
         this.cErrMsg = "WritePrinter error"
         WinAPIEndPagePrinter (this.hPrinter)
         WinAPIEndDocPrinter (this.hPrinter)
         WinAPIClosePrinter (this.hPrinter)
         this.hPrinter = 0
         RETURN false
      ENDIF

      * Check to see if correct number of bytes writen
   RETURN nBytesWritten = cData.length

   PROCEDURE CloseDocument
      IF this.hPrinter = 0
         RETURN false
      ENDIF

      * End the page
      IF .not. WinAPIEndPagePrinter (this.hPrinter)
         this.cErrMsg = "EndPagePrinter error"
         WinAPIEndDocPrinter (this.hPrinter)
         WinAPIClosePrinter (this.hPrinter)
         this.hPrinter = 0
         RETURN false
      ENDIF

      * Inform the spooler that the document is ending
      IF .not. WinAPIEndDocPrinter (this.hPrinter)
         this.cErrMsg = "CloseDocument error"
         WinAPIClosePrinter (this.hPrinter)
         this.hPrinter = 0
         RETURN false
      ENDIF

      * Tidy up the printer handle
      WinAPIClosePrinter (this.hPrinter)
      this.hPrinter = 0
   RETURN true

   PROCEDURE PrintLine (cData)
      IF pcount () >= 1
         this.Print (cData)
      ENDIF
      this.Print (chr (13) + chr (10))
   RETURN

   PROCEDURE Eject
      this.Print (chr (12))
   RETURN

ENDCLASS

#endif

************************************************************************
* eof
************************************************************************
